"use strict";
const { Model } = require("sequelize");

module.exports = (sequelize, DataTypes) => {
  class Account extends Model {
    /**
     * Helper method for defining associations.
     * This method is not a part of Sequelize lifecycle.
     * The `models/index` file will call this method automatically.
     */
    static associate(models) {
      // define association here
      Account.hasMany(models.Location, {
        onDelete: "RESTRICT",
      });
      Account.belongsTo(models.User, {
        foreignKey: {
          name: "createdBy",
          allowNull: false,
        },
        as: "accountCreator",
      });
      Account.belongsTo(models.User, {
        foreignKey: {
          name: "updatedBy",
          allowNull: false,
        },
        as: "accountUpdater",
      });
      Account.hasMany(models.Account, {
        foreignKey: "ParentAccountId",
        as: "parentAccountId",
      });
    }
  }

  Account.init(
    {
      id: {
        type: DataTypes.UUID,
        defaultValue: DataTypes.UUIDV4,
        primaryKey: true,
        allowNull: false,
      },
      name: {
        type: DataTypes.STRING,
        allowNull: false,
      },
      parentAccountName: {
        type: DataTypes.STRING,
      },
      website: {
        type: DataTypes.STRING,
        allowNull: false,
        // checks for url format
        validate: {
          isUrl: true,
        },
      },
      domain: {
        type: DataTypes.STRING,
        unique: true,
        allowNull: false,
      },
      type: {
        type: DataTypes.STRING,
        validate: {
          // type value must be one of these
          isIn: {
            args: [
              [
                "Parent",
                "Subsidiary",
                "Independent",
              ],
            ],
            msg: "Type must be 'Parent', 'Subsidiary' or 'Independent'",
          },
        },
        allowNull: false,
      },
      aliasName: {
        type: DataTypes.JSON,
      },
      scrubbedAliasName: {
        type: DataTypes.ARRAY(DataTypes.STRING),
        allowNull: false,
      },
      email: {
        type: DataTypes.STRING,
        // checks for email format
        validate: {
          isEmail: true,
        },
      },
      industry: {
        type: DataTypes.STRING,
      },
      subIndustry: {
        type: DataTypes.STRING,
      },
      sicCode: {
        type: DataTypes.STRING,
      },
      sicDescription: {
        type: DataTypes.STRING,
      },
      naicsCode: {
        type: DataTypes.INTEGER,
        defaultValue: 0,
      },
      naicsDescription: {
        type: DataTypes.STRING,
      },
      employeeRange: {
        type: DataTypes.STRING,
        validate: {
          // employeeRange value must be one of these
          isIn: {
            args: [
              [
                "0 - 10",
                "11 - 50",
                "51 - 200",
                "201 - 500",
                "501 - 1,000",
                "1,001 - 5,000",
                "5,001 - 10,000",
                "10,000+",
              ],
            ],
            msg: "employeeRange must be '0 - 10','11 - 50','51 - 200','201 - 500','501 - 1,000','1,001 - 5,000','5,001 - 10,000' or '10,000+'",
          },
        },
        allowNull: false,
      },
      employeeSize: {
        type: DataTypes.INTEGER,
        //checks for valid integers
        validate: {
          isInt: true,
        },
        defaultValue: 0,
      },
      employeeSizeLI: {
        type: DataTypes.INTEGER,
        //checks for valid integers
        validate: {
          isInt: true,
        },
        defaultValue: 0,
      },
      employeeSizeZPlus: {
        type: DataTypes.INTEGER,
        //checks for valid integers
        validate: {
          isInt: true,
        },
        defaultValue: 0,
      },
      employeeSizeOthers: {
        type: DataTypes.INTEGER,
        //checks for valid integers
        validate: {
          isInt: true,
        },
        defaultValue: 0,
      },
      revenue: {
        type: DataTypes.FLOAT,
        //checks for valid integers
        validate: {
          isInt: true,
        },
        defaultValue: 0,
      },
      revenueRange: {
        type: DataTypes.STRING,
        validate: {
          // revenueRange value must be one of these
          isIn: {
            args: [
              [
                "0 - $1M",
                "$1M - $10M",
                "$10M - $50M",
                "$50M - $100M",
                "$100M - $250M",
                "$250M - $500M",
                "$500M - $1B",
                "$1B - $10B",
                "$10B+",
              ],
            ],
            msg: "revenueRange must be '0 - $1M', '$1M - $10M', '$10M - $50M', '$50M - $100M', '$100M - $250M', '$250M - $500M', '$500M - $1B', '$1B - $10B' or '$10B+'",
          },
        },
      },
      totalFunding: {
        type: DataTypes.FLOAT,
        //checks for valid integers
        validate: {
          isInt: true,
        },
        defaultValue: 0,
      },
      latestFundingAmount: {
        type: DataTypes.FLOAT,
        //checks for valid integers
        validate: {
          isInt: true,
        },
        defaultValue: 0,
      },
      itSpend: {
        type: DataTypes.FLOAT,
        //checks for valid integers
        validate: {
          isInt: true,
        },
        defaultValue: 0,
      },
      liUrl: {
        type: DataTypes.TEXT,
        // checks for url format
        validate: {
          isUrl: true,
          is: ["linkedin.com/company", "i"],
        },
      },
      description: {
        type: DataTypes.TEXT,
      },
      duns: {
        type: DataTypes.INTEGER,
        //checks for valid integers
        validate: {
          isInt: true,
        },
        defaultValue: 0,
      },
      technology: {
        type: DataTypes.ARRAY(DataTypes.STRING),
        allowNull: false,
      },
      tags: {
        type: DataTypes.ARRAY(DataTypes.STRING),
        allowNull: false,
      },
      state: {
        type: DataTypes.STRING,
      },
      zipCode: {
        type: DataTypes.STRING,
      },
      country: {
        type: DataTypes.STRING,
      },
      disposition: {
        type: DataTypes.STRING,
        defaultValue: 'Active Account',
        validate: {
          // type value must be one of these
          isIn: {
            args: [
              [
                "Active Account",
                "Acquired/Merged/Subsidiary",
                "Bankrupt/Shut Down",
              ],
            ],
            msg: "Disposition must be 'Active Account', 'Acquired/Merged/Subsidiary' or 'Bankrupt/Shut Down'",
          },
        },
      },
      comments: {
        type: DataTypes.TEXT,
      },
      requestedBy: {
        type: DataTypes.STRING,
      },
      foundedYear: {
        type: DataTypes.INTEGER,
        //checks for valid integers
        validate: {
          isInt: true,
        },
        defaultValue: 0,
      },
      lastRaisedAt: {
        type: DataTypes.DATE,
      },
      seoDescription: {
        type: DataTypes.TEXT,
      },
      keywords: {
        type: DataTypes.ARRAY(DataTypes.STRING),
      },
      previousName: {
        type: DataTypes.STRING,
      },
      previousWebsite: {
        type: DataTypes.STRING,
        // checks for url format
        validate: {
          isUrl: true,
        },
      },
      previousType: {
        type: DataTypes.STRING,
        // previousType value must be one of these
        validate: {
          isIn: {
            args: [
              [
                "Parent",
                "Subsidiary",
                "Proprietary",
                "Partnership",
                "Independent",
              ],
            ],
            msg: "Previous-Type must be 'Parent', 'Subsidiary', 'Proprietary', 'Partnership' or 'Independent'",
          },
        },
      },
      previousIndustry: {
        type: DataTypes.STRING,
      },
      previousSubIndustry: {
        type: DataTypes.STRING,
      },
      previousEmployeeSize: {
        type: DataTypes.INTEGER,
      },
      previousRevenue: {
        type: DataTypes.FLOAT,
      },
      previousTechnology: {
        type: DataTypes.ARRAY(DataTypes.STRING),
      },
    },
    {
      sequelize,
      modelName: "Account",
      timestamps: true,
      indexes: [
        {
          fields: ["name"],
        },
        {
          fields: ["website"],
        },
        {
          fields: ["type"],
        },
        {
          fields: ["industry"],
        },
        {
          fields: ["subIndustry"],
        },
        {
          fields: ["sicCode"],
        },
        {
          fields: ["sicDescription"],
        },
        {
          fields: ["naicsCode"],
        },
        {
          fields: ["naicsDescription"],
        },
        {
          fields: ["employeeSize"],
        },
        {
          fields: ["revenue"],
        },
        {
          fields: ["state"],
        },
        {
          fields: ["zipCode"],
        },
        {
          fields: ["country"],
        },
        {
          fields: ["technology"],
          using: "gin",
        },
        {
          fields: ["tags"],
          using: "gin",
        },
        {
          fields: ["scrubbedAliasName"],
          using: "gin",
        },
      ],
    }
  );
  return Account;
};
