const _ = require('lodash');
const dot = require('dot-object');
const { Account,Location } = require("../../index")

const {
    LOCATION_TYPES,
    MILLION,
    BILLION,
    MASTER_GOLDMINE_MAPPING
} = require("./constants");

const excludeMasterAccountField = [
    'id',
    'name',
    'parentAccountName',
    'aliasName',
    'scrubbedAliasName',
    'totalFunding',
    'latestFundingAmount',
    'itSpend',
    'duns',
    'tags',
    'state',
    'zipCode',
    'country',
    'comments',
    'requestedBy',
    'foundedYear',
    'lastRaisedAt',
    'seoDescription',
    'keywords',
    'previousName',
    'previousWebsite',
    'previousType',
    'previousIndustry',
    'previousSubIndustry',
    'previousEmployeeSize',
    'previousRevenue',
    'previousTechnology',
    'createdBy',
    'updatedBy',
    'ParentAccountId',
    'createdAt',
];
const excludeMasterLocationFields = [
    'id',
    'type',
    'stateAbbreviation',
    'timeZone',
    'region',
    'phone2',
    'addressDedupeKey',
    'accountDomain',
    'createdAt',
    'updatedAt',
    'AccountId',
    'createdBy',
    'updatedBy',
];

function splitRangeData(_data) {
    const data = _.cloneDeep(_data);

    if (!data) {
        return [0, 0];
    }

    const regex = /[\$,\s]/g;
    const replacedData = data.replaceAll(regex, '');
    const splittedData = replacedData.split('-');
    
    const splittedDataLength = splittedData.length;

    for (let index = 0; index < splittedDataLength; index += 1) {
        const element = splittedData[index];
        if (element.includes('M')) {
            let replacedM = element.replace('M', '');
            replacedM *= MILLION;
            splittedData[index] = replacedM;
        }
        if (element.includes('B')) {
            let replacedB = element.replace('B', '');
            replacedB *= BILLION;
            splittedData[index] = replacedB;
        }
        if (element.includes('+')) {
            const replacedPlus = element.replace('+', '');
            if (element.includes('B')) {
                let numbers = replacedPlus.replace('B', '');
                numbers *= BILLION;
                return [numbers, null];
            }
            return [replacedPlus, null];
        }
    }
    return splittedData;
}

function mapAccountMasterToGoldMine(_masterAccount) {
    let masterAccount = _.cloneDeep(_masterAccount);

    const processedAccount = {};
    const {
        revenueRange,
        employeeRange,
        'Locations.address1': address1HQ,
        'Locations.address2': address2HQ,
        'Locations.city': cityHQ,
        'Locations.state': stateHQ,
        'Locations.country': countryHQ,
        'Locations.zipCode': zipCodeHQ,
        'Locations.phone1': phoneHQ,
    } = masterAccount;

    const [lowerRevenue, upperRevenue] = splitRangeData(revenueRange);
    masterAccount.lowerRevenue = lowerRevenue;
    masterAccount.upperRevenue = upperRevenue;

    const [lowerEmployeeSize, upperEmployeeSize] = splitRangeData(employeeRange);
    masterAccount.lowerEmployeeSize = lowerEmployeeSize;
    masterAccount.upperEmployeeSize = upperEmployeeSize;
    masterAccount.employeeRange = employeeRange;

    // addressHQ
    masterAccount.addressHQ = {
        address1HQ,
        address2HQ,
        cityHQ,
        stateHQ,
        countryHQ,
        zipCodeHQ,
    };

    // phoneHQ
    masterAccount.phoneHQ = phoneHQ;

    // remove unWanted fields
    delete masterAccount.revenueRange;
    masterAccount = _.omit(masterAccount, [
        'Locations.address1',
        'Locations.address2',
        'Locations.city',
        'Locations.state',
        'Locations.zipCode',
        'Locations.country',
        'Locations.phone1',
    ]);

    const masterAccountKeys = Object.keys(masterAccount);
    console.log("masterAccountKeys : ",masterAccountKeys);
    masterAccountKeys.forEach((key) => {
        processedAccount[MASTER_GOLDMINE_MAPPING[key]] = masterAccount[key];
    });

    return processedAccount;
}

async function getAccountByDomain(domain) {
    const accountData = await Account.findOne({
        where: {
            domain,
        },
        attributes: {
            exclude: excludeMasterAccountField,
        },
        raw: true,
        subQuery: false,
    });
    return accountData;
}

async function getLocationByDomain(domain) {
    const locationData = await Location.findOne({
        where: {
            accountDomain: domain,
            type: LOCATION_TYPES.HEAD_OFFICE
        },
        attributes: {
            exclude: excludeMasterLocationFields,
        },
        order: [
            ['updatedAt', 'desc']
        ],
        raw: true,
        subQuery: false,
    });

    return locationData;
}

async function getAccountAndLocationByDomain(domain) {
    if (!domain) return null;

    const promises = [];
    promises.push(getAccountByDomain(domain));
    promises.push(getLocationByDomain(domain));

    return Promise.all(promises)
        .then((response) => {
            let [account, location] = response;

            // check account is Empty
            if (!account) return null;

            // check location is not Empty
            if (location) {
                location = {
                    Locations: location
                };
                location = dot.dot(dot.object(location));
                account = Object.assign(account, location);
            }

            if (account) {
                return mapAccountMasterToGoldMine(account);
            }
            return account;
        })
        .catch((error) => {
            throw error;
        });
}

module.exports = {
    getAccountAndLocationByDomain
}