/* eslint-disable class-methods-use-this */
const { isEmpty, cloneDeep, isString } = require('lodash');
const { Sequelize } = require('../../index');

const { Op } = Sequelize;

class FilterHandler {
  checkType(value, type) {
    if (type === 'string') return typeof value === 'string';
    if (type === 'array') return Array.isArray(value);
    if (type === 'object') return value.constructor === Object;
    return false;
  }

  /**
   * Filter Validator.
   * @param {Object} filterableColumns - Filterable Columns.
   * @param {Object} filter - Applied Filter.
   */
  validate(filterableColumns, filter) {
    // Check that the type of filter value is correct
    if (filter.constructor !== Object)
      throw new Error(`The filter value type is not an object`);

    if (isEmpty(filter)) return;

    const appliedFilterColumns = Object.keys(filter);

    for (let index = 0; index < appliedFilterColumns.length; index += 1) {
      const appliedFilterColumn = appliedFilterColumns[index];
      if (!filterableColumns[appliedFilterColumn])
        throw new Error(
          `Filter will not be applied across this column {column: ${appliedFilterColumn}}`,
        );
      if (!this.checkType(filter[appliedFilterColumn], 'object'))
        throw new Error(
          `${appliedFilterColumn} Column value type is not an object`,
        );
      if (
        !filterableColumns[appliedFilterColumn].operator.includes(
          filter[appliedFilterColumn].operator,
        )
      )
        throw new Error(`${appliedFilterColumn} column operator is incorrect`);
      if (filter[appliedFilterColumn].operator === 'isNull')
        // eslint-disable-next-line no-continue
        continue;
      if (isEmpty(filter[appliedFilterColumn].value))
        throw new Error(`${appliedFilterColumn} Column value is incorrect`);
      if (
        !this.checkType(
          filter[appliedFilterColumn].value,
          filterableColumns[appliedFilterColumn].type,
        )
      )
        throw new Error(
          `${appliedFilterColumn} column value type is incorrect`,
        );
      if (
        filter[appliedFilterColumn].operator === 'between' &&
        (!Array.isArray(filter[appliedFilterColumn].value) ||
        filter[appliedFilterColumn].value.length !== 2 ||
        !filter[appliedFilterColumn].value[0] ||
        !filter[appliedFilterColumn].value[1])
      )
        throw new Error(`${appliedFilterColumn} Column value is incorrect`);
    }
  }

  /**
   * Build Where Clause.
   * @param {Object} _filterColumnsMapping - Human readable to DB column name Mapping Object.
   * @param {Object} _filter - Applied Filter.
   * @param {Object} _where - Where Clause.
   */
  buildWhereClause(_filterColumnsMapping, _filter, _where) {
    const filterColumnsMapping = _filterColumnsMapping
      ? cloneDeep(_filterColumnsMapping)
      : {};
    const filter = cloneDeep(_filter);
    const where = _where ? cloneDeep(_where) : {};

    const appliedFilterColumns = Object.keys(filter);

    for (let index = 0; index < appliedFilterColumns.length; index += 1) {
      const appliedFilterColumn = appliedFilterColumns[index];
      const dbColumnName =
        filterColumnsMapping[appliedFilterColumn] || appliedFilterColumn;

      switch (filter[appliedFilterColumn].operator) {
        case '=':
          let filterValueWithoutSpace;
          if(typeof filter[appliedFilterColumn].value === 'string')
            filterValueWithoutSpace= filter[appliedFilterColumn].value.replaceAll(' ', '');

          if (
            filterValueWithoutSpace &&
            Number(filter[appliedFilterColumn].value).toString() === 'NaN' &&
            Date.parse(filterValueWithoutSpace).toString() !== 'NaN'
          ) {
            const startDate = new Date(
              new Date(
                filter[appliedFilterColumn].value,
              ).setHours(0, 0, 0, 0)
            );
            const endDate = new Date(
              new Date(
                filter[appliedFilterColumn].value,
              ).setHours(23, 59, 59, 999)
            );

            where[dbColumnName] = {
              [Op.between]: [startDate, endDate],
            };
          } else {
            where[dbColumnName] = filter[appliedFilterColumn].value;
          }
          break;
        case '<':
          where[dbColumnName] = {
            [Op.lt]: filter[appliedFilterColumn].value,
          };
          break;
        case '>':
          where[dbColumnName] = {
            [Op.gt]: filter[appliedFilterColumn].value,
          };
          break;
        case 'between':
          let filterFirstValueWithoutSpace = filter[appliedFilterColumn].value[0];
          if (isString(filterFirstValueWithoutSpace))
            filterFirstValueWithoutSpace = filter[appliedFilterColumn].value[0].replaceAll(' ', '');

          let filterSecondValueWithoutSpace = filter[appliedFilterColumn].value[1];
          if (isString(filterSecondValueWithoutSpace))
            filterSecondValueWithoutSpace = filter[appliedFilterColumn].value[1].replaceAll(' ', '');

          if (
            Number(filter[appliedFilterColumn].value[0]).toString() === 'NaN' &&
            Date.parse(filterFirstValueWithoutSpace).toString() !== 'NaN' &&
            Number(filter[appliedFilterColumn].value[1]).toString() === 'NaN' &&
            Date.parse(filterSecondValueWithoutSpace).toString() !== 'NaN'
          ) {
            const startDate = new Date(
              new Date(
                filter[appliedFilterColumn].value[0],
              ).setHours(0, 0, 0, 0)
            );
            const endDate = new Date(
              new Date(
                filter[appliedFilterColumn].value[1],
              ).setHours(23, 59, 59, 999)
            );
            where[dbColumnName] = {
              [Op.between]: [startDate, endDate],
            };
          } else {
            where[dbColumnName] = {
              [Op.between]: filter[appliedFilterColumn].value,
            };
          }
          break;
        case 'isNull':
          where[dbColumnName] = {
            [Op.or]: [
              {
                [Op.is]: null,
              },
              {
                [Op.eq]: '',
              },
            ],
          };
          break;
        case '>=':
          where[dbColumnName] = {
            [Op.gte]: filter[appliedFilterColumn].value,
          };
          break;
        case '<=':
          where[dbColumnName] = {
            [Op.lte]: filter[appliedFilterColumn].value,
          };
          break;
        default:
          throw new Error(`Could not build where clause for ${filter[appliedFilterColumn].operator} operator`);
      }
    }
    return where;
  }
}
module.exports = FilterHandler;
